﻿/**
* Sample code for the GetNearestPostOffice Canada Post service.
* 
* The GetNearestPostOffice service returns a list of Post Offices 
* near a given location including information such as address and phone number.
*
* This sample is configured to access the Developer Program sandbox environment. 
* Use your development key username and password for the web service credentials.
* 
**/

using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.Security.Authentication;
using System.Net;

namespace GetNearestPostOffice
{
    class GetNearestPostOffice
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username and password are imported from the following file
            // CPCWS_SOAP_PostOffice_DotNet_Samples\SOAP\postoffice\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];

            PostOfficePortTypeClient server = new PostOfficePortTypeClient("PostOfficePort", "https://ct.soa-gw.canadapost.ca/rs/soap/postoffice");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;

            // Create Request
            getnearestpostofficerequest request = new getnearestpostofficerequest();
            request.locale = "FR";
            request.maximum = 3;
            request.maximumSpecified = true;
            request.searchdata = new getnearestpostofficerequestSearchdata();
            request.searchdata.postalcode = "K2B8J6";

            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                getnearestpostofficeresponse response = server.GetNearestPostOffice(request);

                // Retrieve values from response object
                if (response.Item.GetType() == typeof(getnearestpostofficeresponsePostofficelist))
                {
                    // Retrieve values from response object
                    getnearestpostofficeresponsePostofficelist postOfficeList = (getnearestpostofficeresponsePostofficelist)response.Item;
                    foreach (var postOffice in postOfficeList.postoffice)
                    {
                        responseAsString += "Office Name: " + postOffice.name + "\r\n";
                        responseAsString += "Office Id: " + postOffice.officeid + "\r\n";
                        responseAsString += "Address: " + postOffice.address.officeaddress + "\r\n\r\n";
                    }
                    
                }
                else
                {
                    messages msgs = (messages)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);

        }
    }
}
